@@ -6,7 +6,6 @@ use Module::Build::Base;
 use vars qw(@ISA);
 @ISA = qw(Module::Build::Base);
 
-sub need_prelink_c { 1 }
 
 
 =head1 NAME
@@ -62,6 +61,8 @@ sub cull_args {
 
 Use '__' instead of '::'.
 
+=back
+
 =cut
 
 sub manpage_separator {
